/*******************************************************************
 Copyright (C) 2011-2024  Patrick H. E. Foubet - S.E.R.I.A.N.E.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or any
later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>
*******************************************************************/

/* lich.h  */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* les structures du chainage des elts */
#define ELT_NBN 4 /* max 16 */
#define ELT_NBITS 0xF
#define ELT_DENY 0x10
#define ELT_INDEN 0x20
#define ELT_ALLOW 0

struct lib { /* stucture d'un libelle */
   char *lib; /* le libelle */
   struct lib * next; /* adresse du prochain libelle */
};

struct libd { /* stucture d'un libelle double */
   char *lib; /* le libelle */
   char *lib2; /* le 2eme libelle facultatif */
   struct libd * next; /* adresse du prochain libelle */
};

struct elt { /* stucture d'un element */
   int stat; /* status (allow/deny, etc ) */
   int no[ELT_NBN]; /* no des requetes en cours */
   char *url; /* url de l'element */
   struct lib *ip; /* adresses IPv4 de l'element */
   struct lib *ip6; /* adresses IPv6 de l'element */
   struct elt * next; /* adresse du prochain element */
};

#define FIN (struct elt *)NULL
#define VIDE (struct lib *)NULL
#define VIDD (struct libd *)NULL

#define Y(s) (getLibelle(#s))

extern long getLC_SIZE(void);
extern long getLC_SIZEL(void);
extern struct elt ** getFirst(void);
extern pthread_mutex_t * getMutElt(void);

extern struct elt * ajouteElt(char * n, int s);
extern void ajouteIp(struct elt *E, char *ip, int v6);
extern struct elt * getNum(int n);
extern void ajouteAllow(char * u);
extern void ajouteDeny(char * u);
extern int estSousDomaine(char * u, char * d);
extern int addNameCidr(char * u, char *l);
extern int addNameCidr6(char * u, char *l);
extern char * getNameCidr(char * u);
extern char * getNameCidr6(char * u);
extern int inAllow(char * u);
extern int inDeny(char * u);
extern void listeLib(struct lib **L);
extern void listeAllow(void);
extern void listeDeny(void);
extern void ajouteLibs(char * e, char * l);
extern void listeLibs(char *s);
extern char * getLibelle(char * u);

